const std = @import("std");
const time = std.time;
const testing = std.testing;

pub fn lerp(comptime T: type, a: T, b: T, t: T) T {
    return (a + (b - a)) * t;
}

pub const Instant = time.Instant;

var m_app_start: time.Instant = undefined;
var m_update: time.Instant = undefined;
var m_prev_update: time.Instant = undefined;
var m_dt: u64 = undefined;
var m_dtf: f32 = undefined;
var m_smooth_dtf: f32 = undefined;
var m_frame_count: u32 = undefined;

pub fn init() void {
    m_frame_count = 0;
    m_app_start = now();
    m_update = m_app_start;
}

pub fn update() void {
    m_frame_count += 1;
    const now_time = now();
    m_dt = now_time.since(m_prev_update);
    m_prev_update = m_update;
    m_update = now_time;
    m_dtf = @intToFloat(f32, m_dt) / time.ms_per_s;
    m_smooth_dtf = lerp(f32, m_smooth_dtf, m_dtf, (1.0 / 120.0));
}

pub fn deinit() void {}

pub fn now() time.Instant {
    return time.Instant.now() catch unreachable;
}

pub fn lap(tick: *time.Instant) u64 {
    const prev = tick;
    tick.* = now();
    return prev;
}

pub fn getFrameCount() u64 {
    return m_frame_count;
}
pub fn getAppStart() u64 {
    return m_app_start;
}
pub fn getPrevFrame() time.Instant {
    return m_prev_update;
}
pub fn getDelta() u64 {
    return m_dt;
}
pub fn deltaf() f32 {
    return m_dtf;
}
pub fn smoothDeltaf() f32 {
    return m_smooth_dtf;
}

test "time tests" {
    init();
    defer deinit();

    time.sleep(10);

    update();

    try testing.expect(m_app_start.order(m_update) == .lt);

    time.sleep(10);
    update();

    try testing.expect(m_app_start.order(m_update) == .lt);
    try testing.expect(m_prev_update.order(m_update) == .lt);
}
