const vec = @import("../math/vec.zig");
const Vec4f = vec.Vec4f;

const Texture = @import("textures.zig").Texture;
const DiskTexture = @import("textures.zig").DiskTexture;

pub const MatFlags = packed struct {
    emissive: bool = false, // enable emissive term
    sky: bool = false, // lookup emission from sky cubemap
    water: bool = false,
    slime: bool = false,
    lava: bool = false,
    refractive: bool = false, // enable refraction term
    warped: bool = false, // uv animated
    animated: bool = false, // keyframe animated
    underwater: bool = false, // SURF_UNDERWATER
};

pub const Material = packed struct {
    albedo: Texture.Id, // rgba8 srgb (albedo, alpha)
    rome: Texture.Id, // rgba8 srgb (roughness, occlusion, metallic, emission)
    normal: Texture.Id, // rg16 (tangent space xy)
    flags: MatFlags,
    mean_free_path: Vec4f, // .w = scatter dir 'g'
    ior: f32, // index of refraction
    bumpiness: f32,
};

pub const DiskMaterial = packed struct {
    albedo: DiskTexture.Id,
    rome: DiskTexture.Id,
    normal: DiskTexture.Id,
    flags: MatFlags,
    ior: f32,
};
